var NRS = (function(NRS, $) {

	var getForgingAccounts = function(callback) {
		var total = {};
		total.accounts_num = 0;
		total.total_balance = 0;
		total.total_hold = 0;
		total.total_par = 0;
		total.total_daily_par = 0;
		total.total_monthly_par = 0;

		NRS.sendRequest("getForging", {
			"adminPassword": NRS.getAdminPassword()
		}, function(response) {
			var accounts = response.generators;

			var redZone = function (balance, amount, genesis, last, currentTime, untilRedZone, deltaTime, status) {

				if (!status) return 0;

				var newUntilRedZone = untilRedZone + deltaTime;
				var future = newUntilRedZone - currentTime;
				var genesisIncreased = genesis + NRS.getAverageDailyEmission() * future / 864;
				var check = new prizm.Para().ordinary(balance, amount, last, genesisIncreased, future)
							<= new prizm.Para().compound(balance, amount, last, genesisIncreased, status, future);
				if (!deltaTime) {
					if (check) return -1;
					deltaTime = 8640000;
				}
				var newDeltaTime = Math.abs(deltaTime);
				if (newDeltaTime <= 100) {
					return Math.floor(newUntilRedZone) + last;
				} else {
					if (newDeltaTime < 8640000 || check) newDeltaTime = Math.ceil(newDeltaTime / 2);
					if (check) newDeltaTime = -newDeltaTime;
					return redZone(balance, amount, genesis, last, currentTime, newUntilRedZone, newDeltaTime, status);
				}
			}

			if (Array.isArray(accounts)) accounts = accounts.map(function(item) {
				return { "accountRS": item.accountRS };
			});
			for (var index in accounts) {
				NRS.sendRequest("getPara+", {
					"account": accounts[index].accountRS
				}, function(response) {
					if (response.balance) {
						accounts[index].balance = Number(response.balance);
						total.total_balance += Number(response.balance);
						accounts[index].hold = Number(response.hold);
						total.total_hold += Number(response.hold);
						accounts[index].amount = Number(response.amount);
						accounts[index].last = Number(response.last);
						accounts[index].height = Number(response.height);
						accounts[index].holdStatus = NRS.lastBlockHeight >= 1200000 
								&& NRS.lastBlockHeight - Number(response.height) <= 100000 && Number(response.balance) <= 11000000;
						accounts[index].par = Math.floor(new prizm.Para().calc(Number(response.balance), Number(response.amount)
								, Number(response.last), Number(NRS.genesisAmount), accounts[index].holdStatus) * 100);
						total.total_par += accounts[index].par;
						accounts[index].daily_par = Math.floor(new prizm.Para().calc(Number(response.balance), Number(response.amount)
								, Number(response.last), Number(NRS.genesisAmount) - NRS.getAverageDailyEmission() * 100, accounts[index].holdStatus, 86400) * 100) - accounts[index].par;
						total.total_daily_par += accounts[index].daily_par;
						accounts[index].monthly_par = Math.floor(new prizm.Para().calc(Number(response.balance), Number(response.amount)
								, Number(response.last), Number(NRS.genesisAmount) - NRS.getAverageDailyEmission() * 3000, accounts[index].holdStatus, 86400 * 30) * 100) - accounts[index].par;
						total.total_monthly_par += accounts[index].monthly_par;
						var multi = new prizm.Para().multi(response.balance, response.amount);
						accounts[index].paratax = 0;
						if (Number(response.balance) > 0 && multi > 0)
							accounts[index].paratax = Math.floor((1 - accounts[index].daily_par / Number(response.balance) / multi * 100) * 10000);
						var timeFromLastToNow = NRS.formatTimestamp(response.last, false, false, true);
						accounts[index].date_red_zone = redZone(Number(response.balance), Number(response.amount), Math.abs(NRS.genesisAmount)
												, Number(response.last), timeFromLastToNow, timeFromLastToNow, 0, accounts[index].holdStatus);
						if (!accounts[index].holdStatus) {
 						accounts[index].last = Number.MAX_VALUE;
						accounts[index].status = Number.MAX_VALUE;
						}
					}
				}, false);
				
			}
			
			NRS.forgingAccounts.accounts = accounts;
			NRS.forgingAccounts.total = total;

			var sortDataDefault = function() {
				if (!NRS.sortTable.column) {
					NRS.sortTable.column = "account";
					NRS.sortTable.asc = 1;
					NRS.sortTable.reverse = 1;
				}
			}();
			
			callback(forgingAccountsPageLoaded);

		});

	};

	var sortData = function(callback) {

		var sortParameterName = function() {
				switch (NRS.sortTable.column) {
					case "hierarchy":
					   return "amount";
					case "days":
					   NRS.sortTable.reverse = -1;
					   return "last";
					case "para":
					   return "par";
					case "day_para":
					   return "daily_par";
					case "monthly_income":
					   return "monthly_par";
					case "status":
					   NRS.sortTable.reverse = -1;
					   return "height";
					default:
					   return NRS.sortTable.column;
				}
		}();

		var markSortedColumn = function() {
			var arrow = NRS.sortTable.asc * NRS.sortTable.reverse > 0 ? "&#8595;" : "&#8593;";
			var fields = ["account", "balance", "hierarchy", "days", "date_red_zone", "para", "day_para", "monthly_par", "monthly_income", "status"];
			for (var index in fields) {
				$("#" + fields[index]).children(".arrow").html("");
			}
			$("#" + NRS.sortTable.column).children(".arrow").html(arrow);
		}();


		if (Array.isArray(NRS.forgingAccounts.accounts)) {
			var asc = NRS.sortTable.asc;
			NRS.forgingAccounts.total.accounts_num = NRS.forgingAccounts.accounts.length;
			if (sortParameterName === "account") {
				NRS.forgingAccounts.accounts.sort((arg1, arg2) => arg1.accountRS > arg2.accountRS ? asc : -asc);
			} else if (sortParameterName === "balance") {
				NRS.forgingAccounts.accounts.sort((arg1, arg2) => arg1.balance + arg1.hold > arg2.balance + arg2.hold ? asc : -asc);
			} else {
				NRS.forgingAccounts.accounts.sort((arg1, arg2) => arg1[sortParameterName] > arg2[sortParameterName] ? asc : -asc);
			}
			for (var index in NRS.forgingAccounts.accounts) {
    			NRS.forgingAccounts.accounts[index].sequenceNumber = Number(index) + 1;
			}
			callback();
		}
	}

	NRS.setup.forging_accounts = function() {
		var options = {
			"id": 'sidebar_forging_accounts',
			"titleHTML": '<i class="fa fa-briefcase"></i> <span data-i18n="forging_accounts">Forging accounts</span>',
			"page": 'forging_accounts',
			"desiredPosition": 15
		};
		NRS.addSimpleSidebarMenuItem(options);
	};

	NRS.pages.forging_accounts = function() {
		if (NRS.forgingAccounts.accounts) {
			sortData(forgingAccountsPageLoaded);
		} else {
			getForgingAccounts(sortData);
		}
		changeDataVisibility();
	};

	var forgingAccountsPageLoaded = function() {

		if (NRS.forgingAccounts.accounts.length > NRS.itemsPerPage * NRS.pageNumber) NRS.hasMorePages = true;
		
		var pageAccounts = NRS.forgingAccounts.accounts.slice(NRS.pageNumber * NRS.itemsPerPage - NRS.itemsPerPage, NRS.pageNumber * NRS.itemsPerPage);

		var total = NRS.forgingAccounts.total;

		var rows = "";
		
		for (var index in pageAccounts) {

			var account = pageAccounts[index];
			rows += "<tr>" +
				"<td>" + account.sequenceNumber + ".</td>" +
				"<td>" + NRS.getAccountLink(account, "account", undefined, undefined, true, undefined, NRS.hideData) + "</td>" +
				"<td>" + NRS.formatAmount((account.balance)/ 100, undefined, undefined, undefined, NRS.hideData) + "</td>" +
				"<td>" + NRS.formatAmount(account.amount / 100, undefined, undefined, undefined, NRS.hideData) + "</td>" +
				"<td>" + (account.holdStatus ? NRS.formatTimestamp(account.last, false, false, true, true) : 0) + "</td>" +
				"<td>" + (account.holdStatus ? account.date_red_zone < 0 ? "red zone" : NRS.formatTimestamp(account.date_red_zone, true) : "-") + "</td>" +
				"<td>" + NRS.formatAmount(account.par / 100, undefined, undefined, undefined, NRS.hideData) + "</td>" +
				"<td>" + NRS.formatAmount(account.daily_par / 100, undefined, undefined, undefined, NRS.hideData) + "</td>" +
				"<td>" + NRS.formatAmount(account.monthly_par / 100, undefined, undefined, undefined, NRS.hideData) + "</td>" +
				"<td>" + NRS.calculatePercentage(account.monthly_par, account.balance) + "%" + "</td>" +
				"<td>" + NRS.formatAmount(account.holdStatus ? NRS.lastBlockHeight - account.height : 0) + "</td>" +
			"</tr>";
		}

		NRS.dataLoaded(rows);

		if (Number(NRS.settings['theme'])) {
			$("#total_row").css('background-color', '#302a3e');
		} else {
			$("#total_row").css('background-color', '#f7f8ff');
		}

		$("#accounts_num").html(NRS.hideData ? "**" :total.accounts_num);
		$("#total_coins").html(NRS.hideData ? "*** ***" : NRS.formatStyledAmount((total.total_balance + total.total_hold + total.total_par) / 100));
		$("#total_balance").html(NRS.hideData ? "*** ***" : NRS.formatStyledAmount((total.total_balance) / 100));
		$("#total_par").html(NRS.hideData ? "*** ***" :NRS.formatStyledAmount(total.total_par / 100));
		$("#total_daily_par").html(NRS.hideData ? "*** ***" :NRS.formatStyledAmount(total.total_daily_par / 100));
		$("#total_monthly_par").html(NRS.hideData ? "*** ***" :NRS.formatStyledAmount(total.total_monthly_par / 100));
		$("#total_monthly_income").html(NRS.calculatePercentage(total.total_monthly_par, total.total_balance) + "%");
	};

	$("#file_selector_button1, #file_selector_button2").on("click", function() {
	
        $("#file_selector").click();
    });
	$("#hide_data").on("click", function() {
	
        NRS.hideData = !NRS.hideData;
		changeDataVisibility();
    });

	var changeDataVisibility = function() {
    if (NRS.hideData) {
        $("#hide_data").html('<i class="ion-eye-disabled"></i>');
    } else {
         $("#hide_data").html('<i class="ion-eye"></i>');
    }
    $("#sidebar_account_id").html("PRIZM-<br/>"+String(NRS.hideData ? "PRIZM-****-****-****-*****" : NRS.accountRS).escapeHTML().substring(6));
  $("#account_balance, #account_balance_sidebar").html(NRS.formatStyledAmount(NRS.accountInfo.unconfirmedBalanceNQT, undefined, NRS.hideData));
    sortData(forgingAccountsPageLoaded);
}

	document.getElementById("file_selector").addEventListener("change", (event) => {

		const reader = new FileReader();
		reader.onload = function () {
			const pass = reader.result.split("\n");
			for (var index in pass) {
				if (pass[index]) startForg(pass[index].replace(/[^\x20-\x7E]/g, ''));
			}
		}
		reader.readAsText(event.target.files[0]);	
	});

	var startForg = function(pass) {

		NRS.sendRequest("getGuaranteedBalance", {
			"account": NRS.getAccountId(pass),
			"numberOfConfirmations": 1440
		}, function (response) {
			if (Number(response.guaranteedBalanceNQT) >= 100000) {
				NRS.sendRequest("startForging", { "secretPhrase": pass }, function(response) { console.log(response); }, false, true);
			}
		});
	}

	return NRS;
}(NRS || {}, jQuery));
